/*
This is XTM4XMLDB, a TMAPI implementation for native XMLdb's
Copyright (C) 2003  Stefan Lischke
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
 
This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.
 
You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.
 
Stefan Lischke, hereby disclaims all copyright interest in
the library 'XTM4XMLDB' (a TMAPI implementation).
 
Stefan Lischke (lischke@novacom.net)
 */

package org.xtm4xmldb.core;

import org.tmapi.core.TopicMap;
import org.tmapi.core.TMAPIException;
import org.xmldb.api.base.*;
import org.xmldb.api.modules.*;
import org.xmldb.api.*;

import org.xtm4xmldb.utils.*;
import org.xtm4xmldb.utils.*;
import org.xtm4xmldb.index.core.*;

import org.tmapi.core.*;
import org.tinyTIM.TopicMapImpl;
import org.tinyTIM.LocatorImpl;
import org.tinyTIM.abstractTopicMap;

import java.util.Set;

import org.tmapiutils.impexp.xtm.XTMSerializer;
import org.tinyTIM.usefulStuff;
public class xtm4xmldb_TopicMap extends abstractTopicMap {
    private Collection              col;
    private Collection              assoc_col;
    private Collection              topic_col;
    
    private XPathQueryService               topic_xpathservice;
    private XPathQueryService               assoc_xpathservice;
    private CollectionManagementService     colservice;
    
    public runtimeParser parser;
    public XTMSerializer serializer;
    
    /**
     * Constructor for creating a TopicMap, which creates the collection
     * @param tms
     * @param colname
     * @throws XMLDBException
     *
     */
    public xtm4xmldb_TopicMap(xtm4xmldb_TopicMapSystem tms,String colname)throws XMLDBException,TMAPIException{
        super(tms,new LocatorImpl(colname));
        System.out.println("created TopicMap "+getObjectId());
        //creating collection and delete if there was one
         this.col        =   ((xtm4xmldb_TopicMapSystem)this.tmSystem).getCollectionManagementService().createCollection(colname);
         ((xtm4xmldb_TopicMapSystem)this.tmSystem).getCollectionManagementService().removeCollection(colname);
        
        this.col        =   ((xtm4xmldb_TopicMapSystem)this.tmSystem).getCollectionManagementService().createCollection(colname);
        this.topic_col        =   ((xtm4xmldb_TopicMapSystem)this.tmSystem).getCollectionManagementService().createCollection(colname);
        this.assoc_col        =   ((xtm4xmldb_TopicMapSystem)this.tmSystem).getCollectionManagementService().createCollection(colname);
        
        //this collectionmanagementservice
        
        this.colservice     =   (CollectionManagementService)this.col.getService("CollectionManagementService", "1.0");
        
        //creating subcollections
        topic_col      =   this.colservice.createCollection("topics");
        assoc_col      =   this.colservice.createCollection("assocs");
      //creating Xpathservices
        try{
            this.topic_xpathservice   =   (XPathQueryService)topic_col.getService("XPathQueryService","1.0");
            this.topic_xpathservice.setNamespace("xlink", "http://www.w3.org/1999/xlink");
            
            this.assoc_xpathservice   =   (XPathQueryService)assoc_col.getService("XPathQueryService","1.0");
            this.assoc_xpathservice.setNamespace("xlink", "http://www.w3.org/1999/xlink");
        }catch(XMLDBException e){
            e.printStackTrace();
        }
    }
    
    
    
    /**
     * Constructor for opening a TopicMap, which parses the content
     * @param tms
     * @param col
     * @throws XMLDBException
     *
     */
    
    public xtm4xmldb_TopicMap(xtm4xmldb_TopicMapSystem tms,Collection col)throws XMLDBException,TMAPIException{
        super(tms,new LocatorImpl(col.getName().substring(col.getName().lastIndexOf("/")+1)));
        this.col        =   col;
        System.out.println("reread TopicMap "+getObjectId());
        //this collectionmanagementservice
        this.colservice     =   (CollectionManagementService)this.col.getService("CollectionManagementService", "1.0");
        
        topic_col      =   this.col.getChildCollection("topics");
        assoc_col      =   this.col.getChildCollection("assocs");
      //creating Xpathservices
        try{
            this.topic_xpathservice   =   (XPathQueryService)topic_col.getService("XPathQueryService","1.0");
            this.topic_xpathservice.setNamespace("xlink", "http://www.w3.org/1999/xlink");
            
            this.assoc_xpathservice   =   (XPathQueryService)assoc_col.getService("XPathQueryService","1.0");
            this.assoc_xpathservice.setNamespace("xlink", "http://www.w3.org/1999/xlink");
        }catch(XMLDBException e){
            e.printStackTrace();
        } 
    }
    
    public void setHelpers(){
  
        this.serializer = new XTMSerializer();
        this.serializer.setProperty(XTMSerializer.PROPERTY_ALLIDS, "true");
        this.parser = new runtimeParser(this.tmSystem);
                
        this.helpers= new java.util.HashMap();
        this.helpersMapping= new java.util.HashMap();
        this.helpersMapping.put("org.tmapi.index.core.TopicsIndex", "org.xtm4xmldb.index.core.xmldb_TopicsIndex");
        this.helpersMapping.put("org.tmapi.index.core.AssociationsIndex", "org.xtm4xmldb.index.core.xmldb_AssociationsIndex");
        this.helpersMapping.put("org.tmapi.index.core.OccurrencesIndex", "org.xtm4xmldb.index.core.xmldb_OccurrencesIndex");
        //        this.helpersMapping.put("org.tmapi.index.core.TopicNamesIndex", "org.tinyTIM.index.core.TopicNamesIndexImpl");
        //        this.helpersMapping.put("org.tmapi.index.core.ScopedObjectsIndex", "org.tinyTIM.index.core.ScopedObjectsIndexImpl");
        //        this.helpersMapping.put("org.tmapi.index.core.AssociationRolesIndex", "org.tinyTIM.index.core.AssociationRolesIndexImpl");
        //        this.helpersMapping.put("org.tmapi.index.core.TopicMapObjectsIndex", "org.tinyTIM.index.core.TopicMapObjectsIndexImpl");
    }
    
    /**
     *
     * Creates a new association in the topic map.
     * The newly created association has no type, no roles and
     * is in the unconstrained scope.
     * @return The newly created Association.
     */
    public Association createAssociation() {
        xtm4xmldb_Association assoc = new xtm4xmldb_Association(this);
        assoc.sync();
        return(assoc);
    }
    
    /**
     * Creates a new topic in the topic map.
     * The newly created topic has no types, names or occurrences.
     * @return The newly created Topic.
     */
    public Topic createTopic() {
        xtm4xmldb_Topic t = new xtm4xmldb_Topic(this);
        return(t);
    }
    
    
    
    /**
     * Returns all of the associations in the topic map.
     * @return an unmodifiable set of Association objects.
     */
    
    public Set getAssociations() {
        return(new xmldb_AssociationSet(this));
    }
    
    /**
     *
     * Returns all of the topics in the topic map.
     * @return an unmodifiable set of Topic objects.
     *
     */
    public Set getTopics() {
        System.out.println("getTopics");
        return(new xmldb_TopicSet(this));
    }
    
    
    
    /**
     *
     * @throws <{TMAPIException}>
     *
     */
    
    public void remove() throws TMAPIException {
        try{
            //System.out.println("remove() "+ this.col.getName()+" # of TopicMaps "+this.tms.getBaseLocators().size());
            ((xtm4xmldb_TopicMapSystem)super.tmSystem).getCollectionManagementService().removeCollection(this.col.getName());
            //System.out.println(" # of TopicMaps after removal"+this.tms.getBaseLocators().size());
        }catch(XMLDBException e){
            e.printStackTrace();
            throw(new TMAPIException("was XMLDBException"+ e.getMessage()));
        }
        
    }
    
    
    
    
    
    /*
     *  xmldb methods
     *
     */
    
    /**
     * returns the collection of this topicmap
     * @return Collection
     */
    
    public Collection getCollection(){
        return(this.col);
    }
    public Collection getTopicCollection(){
        return(this.topic_col);
    }
    public Collection getAssociationCollection(){
        return(this.assoc_col);
    }
    
    
    /*
     *  xtm4xmldb   ID stuff
     * every Topic and Association is represented by an xml snippet
     * inside the topicmap collection.
     * the objectID is the id of this xml snippet
     * it never changes at runtime
     *
     * the objectID of occurrences, topicnames.....
     * all elements that are childs of the first element(assoc,topic)
     * ??????????
     */
    
    
    /**
     * returns the XPathQueryService for the topic collection
     * @return XPathQueryService
     */
    public XPathQueryService getTopicXPathQueryService(){
        return(this.topic_xpathservice);
    }
    /**
     * returns the XPathQueryService for the assoc collection
     * @return XPathQueryService
     */
    public XPathQueryService getAssociationXPathQueryService(){
        return(this.assoc_xpathservice);
    }
    
    public TopicMapObject getObjectById(String id){
        System.out.println("!!getObjectByID() "+id);
        try{
            //first check if it is a topic
            Resource res    =   getTopicCollection().getResource(id);
            if(res!=null)
                return(new xtm4xmldb_Topic(this,res));
            //is there an assoc
            res    =   getAssociationCollection().getResource(id);
            if(res!=null)
                return(new xtm4xmldb_Association(this,res));            
            //check if its an occurrence or a BaseName
            System.out.println("????? now search in the inner tags");
            return (xmldb_XPathTopicMapObject.getById(this,id));
        }catch(XMLDBException e){
            throw new TMAPIRuntimeException(e);
        }        
    }
    
    public void close(){
        //write xtm to collection
    }
    /*
     *------- IndexHandler-----
     *
     */
    public void subscribe(TopicMapObject o){
    }
    public void unsubscribe(TopicMapObject o){
    }
    
    public boolean addSourceLocator(Topic t, Locator loc,boolean test) throws DuplicateSourceLocatorException{
       if(test==true)
            return(testSourceLocator(t,loc));
        return true;
    }
    public void addSourceLocator(TopicMapObject o, Locator loc, boolean test) throws DuplicateSourceLocatorException{
    }

    public void removeSourceLocator(TopicMapObject o, Locator loc){
    }
    
    public boolean addSubjectLocator(Topic t, Locator loc,boolean test) {
        if(test==true)
            return(testSubjectLocator(t,loc));
        return true;
    }
    public void removeSubjectLocator(Topic t, Locator loc){
    }
    public void removeSubjectLocators(Topic t, Set locs){
    }
    
    public boolean addSubjectIdentifier(Topic t, Locator loc,boolean test) {
        if(test==true)
            return(testSubjectIdentifier(t,loc));
        return true;
    }       
    public void removeSubjectIdentifier(Topic t, Locator loc){
    }
    
    public void removeSubjectIdentifiers(Topic t, Set locs){
    }
    
    public Topic getTopicBySubjectIdentifier(Locator loc) {
        //xpath
        return(xmldb_XPathTopic.get("/topic[subjectIdentity/subjectIndicatorRef/@xlink:href='"+loc.getReference()+"']",this));
    }
    
    public Topic getTopicBySubjectLocator(Locator loc) {
        //xpath
        return(xmldb_XPathTopic.get("/topic[subjectIdentity/resourceRef/@xlink:href='"+loc.getReference()+"']",this));
    }
    
    public TopicMapObject getTopicMapObjectBySourceLocator(Locator loc) {
        //xpath
        return(xmldb_XPathTopicMapObject.getBySourceLocator(this,loc));
    }

    public void hasChanged(AssociationRole ar) {
        ((xtm4xmldb_Association)ar.getAssociation()).sync();        
    }
    public void hasChanged(Association a) {
    }
    
    public void hasChanged(Topic t) {
    }
        
    public void hasChanged(TopicName tn) {
        ((xtm4xmldb_Topic)tn.getTopic()).sync();
    }
    
    public void hasChanged(Occurrence oc) {
        ((xtm4xmldb_Topic)oc.getTopic()).sync();    }
    
    public void remove(Topic t) {
    }
    
    public void remove(Variant v) {
    }
    
    public void remove(Association a) {
    }
    
    public void remove(AssociationRole ar) {
    }
    
    public void remove(TopicName tn) {
    }
    
    public void remove(Occurrence oc) {
    }
    

}

