/*
This is XTM4XMLDB, a TMAPI implementation for native XMLdb's
Copyright (C) 2003  Stefan Lischke

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
 
This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.
 
You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.
 
Stefan Lischke, hereby disclaims all copyright interest in
the library 'XTM4XMLDB' (a TMAPI implementation).
 
Stefan Lischke (lischke@novacom.net)
 */
package org.xtm4xmldb.core;

import org.tmapi.core.TopicMapSystem;
import org.tmapi.core.Locator;
import org.tmapi.core.TMAPIException;

import org.xmldb.api.base.*;
import org.xmldb.api.modules.*;
import org.xmldb.api.*;

import org.xtm4xmldb.utils.*;

import org.tinyTIM.TopicMapSystemImpl;
import org.tinyTIM.FeatureSupport;
import java.util.Properties;

public class xtm4xmldb_TopicMapSystem extends TopicMapSystemImpl implements TopicMapSystem {
    private Database db;
    private String start_col;
    private String db_uri;
    private String username;
    private String passwd;
    private CollectionManagementService col_manager;

    public xtm4xmldb_TopicMapSystem(Database db, Properties props,FeatureSupport[] features) throws XMLDBException{
        super(props,features);
        this.db         =   db;
        this.db_uri     =   props.getProperty("org.xtm4xmldb.xmldbURI");
        this.username   =   props.getProperty("org.xtm4xmldb.username");
        this.passwd     =   props.getProperty("org.xtm4xmldb.password");
        
        //getting start collection 
        this.start_col  =   props.getProperty("org.xtm4xmldb.startCol");
        //check if it exists already
        
        Collection col_start_col = getStartCol();
        if(col_start_col==null){
            System.out.println("xtm4xmldb Collection does not exist, create Collection : "+ start_col);
            //create start_col
            Collection maindb   =   db.getCollection(this.db_uri,this.username,this.passwd);
            CollectionManagementService maindbcolman    =  (CollectionManagementService) maindb.getService("CollectionManagementService","1.0");
            col_start_col = maindbcolman.createCollection(start_col);
        }
        
        this.col_manager=   (CollectionManagementService)col_start_col.getService("CollectionManagementService","1.0");
    }
    
    
    
    public org.tmapi.core.TopicMap createTopicMap(String baseLocatorAddress) {
        xtm4xmldb_TopicMap  tmap    =   null;
        try{
            //create TopicMap
            tmap    =   new xtm4xmldb_TopicMap(this,baseLocatorAddress);
        }catch(Exception e){  
            e.printStackTrace(); 
        }
        return(tmap); 
    }
    
    
    
    public org.tmapi.core.TopicMap createTopicMap(String baseLocatorNotation, String baseLocatorAddress) {
        xtm4xmldb_TopicMap  tmap    =   null;
        try{
            //create TopicMap
            tmap    =   new xtm4xmldb_TopicMap(this,baseLocatorAddress);
        }catch(Exception e){
            e.printStackTrace();
        }
        return(tmap);
    }
    
    public org.tmapi.core.TopicMap getTopicMap(Locator baseLocator) {
        return(getTopicMap(baseLocator.getReference()));
    }
    
    public org.tmapi.core.TopicMap getTopicMap(String baseLocatorAddress) {
        xtm4xmldb_TopicMap  tmap;
        try{
            Collection col  =   getStartCol().getChildCollection(baseLocatorAddress);
            if(col==null)  
                return(null);
            //return instance
            tmap    =   new xtm4xmldb_TopicMap(this,col);   
        }catch(Exception e){
            e.printStackTrace();
            return(null);  
        }
        System.out.println("getTopicMap( "+baseLocatorAddress+" )"+tmap);
        return(tmap);    
    }
    
    
    
    public org.tmapi.core.TopicMap getTopicMap(String baseLocatorNotation, String baseLocatorAddress) {
        return(getTopicMap(baseLocatorAddress));
    }
    
    public java.util.Set getBaseLocators() { 
        return(new xmldb_BaseLocatorSet(this)); 
    }

    
    /*
     
     *  
     *  xmldb methods
     *
     */
    public Collection getStartCol()throws XMLDBException{
        return(this.db.getCollection(this.db_uri+"/"+this.start_col,this.username,this.passwd));  
    }

    public CollectionManagementService getCollectionManagementService(){
        return(this.col_manager);
    }

    public Collection getCollection(String colname) throws XMLDBException{
        return(this.db.getCollection(this.db_uri+"/"+this.start_col+colname,this.username,this.passwd));
    }
    
    public void close() {
    }
    
}

